/*
 *  Fera Daniel <feradaniel21@gmail.com>
 *  Grupa: 333CC
 */

#include "menu.h"
#include "lcd.h"

#include <avr/pgmspace.h>
#include <stdio.h>
#include <string.h>

const char str_on[] PROGMEM = "ON";
const char str_off[] PROGMEM = "OFF";
const char str_state[] PROGMEM = "State: ";
const char str_delay[] PROGMEM = "Delay: ";
const char str_depth[] PROGMEM = "Depth: ";
const char str_rate[] PROGMEM = "Rate: ";
const char str_drive[] PROGMEM = "Drive: ";
const char str_tresh[] PROGMEM = "Treshold: ";
const char str_kmin[] PROGMEM = "Kmin: ";
const char str_release[] PROGMEM = "Release: ";
const char str_attack[] PROGMEM = "Attack: ";
const char str_volume[] PROGMEM = "Volume: ";
const char str_gain[] PROGMEM = "Gain: ";

#define FLOAT_INCREMENT 0.05f
#define TIMER_1HZ 15625

char buffer[17];

/* Effects presets & settings */
enum effects {
    EFFECT_DISTORTION = 0,
    EFFECT_TREMOLO,
    EFFECT_FLANGER,
    EFFECT_ECHO,
    EFFECT_COMPRESSION
};

extern uint16_t active_effects;

//DISTORTION
extern float es_distortion_depth;
extern float es_distortion_gain;

//TREMOLO
extern float es_tremolo_depth;
extern uint16_t es_tremolo_rate_Hz;

//FLANGER
extern float es_flanger_delay;
extern float es_flanger_depth;
extern uint16_t es_flanger_rate_Hz;
extern uint16_t timer0_top;

//ECHO
// number between 0 and MAX_OUTPUT_HISTORY - 1
extern uint16_t es_echo_delay;
extern float es_echo_depth;

//COMPRESSION
extern float es_compression_knee_treshold;
extern float es_compression_kmin;
extern float es_compression_alpha_release;
extern float es_compression_alpha_attack;

/* ------------------------------------ */

enum MENU_STATE {
    STATE_WELCOME,
    STATE_ALL_ENTRIES,
    STATE_ABOUT,

    // effects
    STATE_DISTORTION,
    STATE_TREMOLO,
    STATE_FLANGER,
    STATE_ECHO,
    STATE_COMPRESSION
};


static void state_welcome_start();
static void state_welcome_hnd(enum MENU_INPUT input);

static void state_all_entries_start(uint8_t reset);
static void state_all_entries_hnd(enum MENU_INPUT input);

static void state_distortion_start();
static void state_distortion_hnd(enum MENU_INPUT input);
static void state_tremolo_start();
static void state_tremolo_hnd(enum MENU_INPUT input);
static void state_flanger_start();
static void state_flanger_hnd(enum MENU_INPUT input);
static void state_echo_start();
static void state_echo_hnd(enum MENU_INPUT input);
static void state_compression_start();
static void state_compression_hnd(enum MENU_INPUT input);

static volatile enum MENU_STATE g_state;

void MENU_init()
{
    LCD_init();
    LCD_toggle_backlight();
    state_welcome_start();
}

void MENU_updatestate(enum MENU_INPUT input)
{
    // propagate input to state handler
    switch (g_state) {
        case STATE_WELCOME:
            state_welcome_hnd(input);
           break; 
        case STATE_ALL_ENTRIES:
            state_all_entries_hnd(input);
            break;
        case STATE_ABOUT:
            state_all_entries_start(0);
            break;
        case STATE_DISTORTION:
            state_distortion_hnd(input);
            break;
        case STATE_TREMOLO:
            state_tremolo_hnd(input);
            break;
        case STATE_FLANGER:
            state_flanger_hnd(input);
            break;
        case STATE_ECHO:
            state_echo_hnd(input);
            break;
        case STATE_COMPRESSION:
            state_compression_hnd(input);
            break;
    }
}

static void state_welcome_start()
{
    g_state = STATE_WELCOME;
    LCD_writeInstr(LCD_INSTR_clearDisplay);
    LCD_printAt(0, "   Bun venit,   ");
    LCD_printAt(LCD_INSTR_nextLine, "  cantareti! :) ");
}

static void state_welcome_hnd(enum MENU_INPUT input)
{
    switch (input) {
        case MENU_INPUT_OK:
            state_all_entries_start(1);
            break;
        case MENU_INPUT_BACK:
        case MENU_INPUT_UP:
        case MENU_INPUT_DOWN:
            break;
    }
}

#define NUM_ENTRIES 6

static const char *state_all_entries_entries[] = {
    "-> Despre mine",
    "-> Distortion",
    "-> Tremolo",
    "-> Flanger",
    "-> Echo",
    "-> Compression"
};

static int8_t state_all_entries_pos;

static void state_all_entries_start(uint8_t reset)
{
    g_state = STATE_ALL_ENTRIES;
    LCD_writeInstr(LCD_INSTR_clearDisplay);
    LCD_printAt(0, "==Procesor DSP==");
    if (reset)
        state_all_entries_pos = 0;
    LCD_printAt(LCD_INSTR_nextLine, state_all_entries_entries[state_all_entries_pos]);
}

static void state_all_entries_hnd(enum MENU_INPUT input)
{
    switch (input) {
        case MENU_INPUT_OK:
            if (state_all_entries_pos == 0) {
                g_state = STATE_ABOUT;
                LCD_writeInstr(LCD_INSTR_clearDisplay);
                LCD_printAt(0, "=(C)Fera Daniel=");
                LCD_printAt(LCD_INSTR_nextLine, "==Grupa: 333CC==");
            } else if (state_all_entries_pos == 1)
                state_distortion_start();
            else if (state_all_entries_pos == 2)
                state_tremolo_start();
            else if (state_all_entries_pos == 3)
                state_flanger_start();
            else if (state_all_entries_pos == 4)
                state_echo_start();
            else if (state_all_entries_pos == 5)
                state_compression_start();
            break;
        case MENU_INPUT_BACK:
            state_welcome_start();
            break;
        case MENU_INPUT_UP:
            state_all_entries_pos--;
            if (state_all_entries_pos == -1)
                state_all_entries_pos = NUM_ENTRIES - 1;
            LCD_printAt(LCD_INSTR_nextLine, LCD_EMPTY_LINE);
            LCD_printAt(LCD_INSTR_nextLine, state_all_entries_entries[state_all_entries_pos]);
            break;
        case MENU_INPUT_DOWN:
            state_all_entries_pos++;
            if (state_all_entries_pos == NUM_ENTRIES)
                state_all_entries_pos = 0;
            LCD_printAt(LCD_INSTR_nextLine, LCD_EMPTY_LINE);
            LCD_printAt(LCD_INSTR_nextLine, state_all_entries_entries[state_all_entries_pos]);
            break;
    }
}

static int8_t option_pos = 0;
static int8_t option_selected = -1;

static void increment_float(float *x, float incr, float mi, float ma)
{
    *x += incr;
    if (*x > ma)
        *x = ma;
    if (*x < mi)
        *x = mi;
}

static void increment_uint(uint16_t *x, uint16_t incr, uint16_t mi, uint16_t ma)
{
    *x += incr;
    if (*x > ma)
        *x = ma;
    if (*x < mi)
        *x = mi;
}

#define DISTORTION_OPTIONS 3
PGM_P const state_distortion_entries[] PROGMEM = {
    str_state,
    str_gain,
    str_depth
};

static void state_distortion_print_option()
{
    uint8_t offset = 2 * (option_pos == option_selected);
    strcpy_P(buffer, (PGM_P)pgm_read_word(&(state_distortion_entries[option_pos])));
    offset += strlen(buffer);

    LCD_printAt(LCD_INSTR_nextLine, LCD_EMPTY_LINE);
    LCD_printAt(LCD_INSTR_nextLine, "> ");
    LCD_printAt(LCD_INSTR_nextLine + 2 * (option_pos == option_selected), buffer);
    if (option_pos == 0) {
        if (active_effects & _BV(EFFECT_DISTORTION)) {
            strcpy_P(buffer, str_on);
            LCD_printAt(LCD_INSTR_nextLine + offset, buffer);
        }
        else {
            strcpy_P(buffer, str_off);
            LCD_printAt(LCD_INSTR_nextLine + offset, buffer);
        }
    } else if (option_pos == 1) {
        sprintf(buffer, "%d", (uint16_t)(es_distortion_gain));
        LCD_printAt(LCD_INSTR_nextLine + offset, buffer);
    } else if (option_pos == 2) {
        sprintf(buffer, "%d%%", (uint16_t)(es_distortion_depth * 100));
        LCD_printAt(LCD_INSTR_nextLine + offset, buffer);
    }
}

static void state_distortion_start()
{
    LCD_writeInstr(LCD_INSTR_clearDisplay);
    LCD_printAt(0, "===Distortion===");
    g_state = STATE_DISTORTION;
    option_pos = 0;
    option_selected = -1;
    state_distortion_print_option();
}

static void state_distortion_hnd(enum MENU_INPUT input)
{
    switch (input) {
        case MENU_INPUT_OK:
            if (option_selected != -1)
                option_selected = -1;
            else
                option_selected = option_pos;
            state_distortion_print_option();
            break;
        case MENU_INPUT_BACK:
            if (option_selected != -1) {
                option_selected = -1;
                state_distortion_print_option();
            } else
                state_all_entries_start(0);
            break;
        case MENU_INPUT_UP:
            if (option_selected == -1) {
                option_pos--;
                if (option_pos == -1)
                    option_pos++;
            } else {
                if (option_selected == 0) {
                    active_effects ^= _BV(EFFECT_DISTORTION);
                } else if (option_selected == 1)
                    increment_float(&es_distortion_gain, 
                        -2.0f, 0.0f, 50.0f);
                else if (option_selected == 2)
                    increment_float(&es_distortion_depth, 
                        -FLOAT_INCREMENT, 0.0f, 1.0f);
            }
            state_distortion_print_option();
            break;
        case MENU_INPUT_DOWN:
            if (option_selected == -1) {
                option_pos++;
                if (option_pos == DISTORTION_OPTIONS)
                    option_pos--;
            } else {
                if (option_selected == 0) {
                    active_effects ^= _BV(EFFECT_DISTORTION);
                } else if (option_selected == 1)
                    increment_float(&es_distortion_gain, 
                        2.0f, 0.0f, 50.0f);
                else if (option_selected == 2)
                    increment_float(&es_distortion_depth, 
                        FLOAT_INCREMENT, 0.0f, 1.0f);
            }
            state_distortion_print_option();
    }
}

#define TREMOLO_OPTIONS 3
PGM_P const state_tremolo_entries[] PROGMEM = {
    str_state,
    str_depth,
    str_rate
};


static void state_tremolo_print_option()
{
    uint8_t offset = 2 * (option_pos == option_selected);
    strcpy_P(buffer, (PGM_P)pgm_read_word(&(state_tremolo_entries[option_pos])));
    offset += strlen(buffer);

    LCD_printAt(LCD_INSTR_nextLine, LCD_EMPTY_LINE);
    LCD_printAt(LCD_INSTR_nextLine, "> ");
    LCD_printAt(LCD_INSTR_nextLine + 2 * (option_pos == option_selected), buffer);

    if (option_pos == 0) {
        if (active_effects & _BV(EFFECT_TREMOLO)) {
            strcpy_P(buffer, str_on);
            LCD_printAt(LCD_INSTR_nextLine + offset, buffer);
        }
        else {
            strcpy_P(buffer, str_off);
            LCD_printAt(LCD_INSTR_nextLine + offset, buffer);
        }
    } else if (option_pos == 1) {
        sprintf(buffer, "%d%%", (uint16_t)(es_tremolo_depth * 100));
        LCD_printAt(LCD_INSTR_nextLine + offset, buffer);
    } else if (option_pos == 2) {
        sprintf(buffer, "%dHz", es_tremolo_rate_Hz);
        LCD_printAt(LCD_INSTR_nextLine + offset, buffer);
    }
}

static void state_tremolo_start()
{
    LCD_writeInstr(LCD_INSTR_clearDisplay);
    LCD_printAt(0, "=====Tremolo====");
    g_state = STATE_TREMOLO;
    option_pos = 0;
    option_selected = -1;
    state_tremolo_print_option();
}

static void state_tremolo_hnd(enum MENU_INPUT input)
{
    switch (input) {
        case MENU_INPUT_OK:
            if (option_selected != -1)
                option_selected = -1;
            else
                option_selected = option_pos;
            state_tremolo_print_option();
            break;
        case MENU_INPUT_BACK:
            if (option_selected != -1) {
                option_selected = -1;
                state_tremolo_print_option();
            } else
                state_all_entries_start(0);
            break;
        case MENU_INPUT_UP:
            if (option_selected == -1) {
                option_pos--;
                if (option_pos == -1)
                    option_pos++;
            } else {
                if (option_selected == 0) {
                    active_effects ^= _BV(EFFECT_TREMOLO);
                } else if (option_selected == 1)
                    increment_float(&es_tremolo_depth, 
                        -FLOAT_INCREMENT, 0.0f, 1.0f);
                else if (option_selected == 2) {
                    increment_uint(&es_tremolo_rate_Hz, -1, 0, 16);
                    OCR1A = TIMER_1HZ / es_tremolo_rate_Hz;
                }
            }
            state_tremolo_print_option();
            break;
        case MENU_INPUT_DOWN:
            if (option_selected == -1) {
                option_pos++;
                if (option_pos == TREMOLO_OPTIONS)
                    option_pos--;
            } else {
                if (option_selected == 0) {
                    active_effects ^= _BV(EFFECT_TREMOLO);
                } else if (option_selected == 1)
                    increment_float(&es_tremolo_depth, 
                        FLOAT_INCREMENT, 0.0f, 1.0f);
                else if (option_selected == 2) {
                    increment_uint(&es_tremolo_rate_Hz, 1, 0, 16);
                    OCR1A = TIMER_1HZ / es_tremolo_rate_Hz;
                }
            }
            state_tremolo_print_option();
            break;
    }
}

#define FLANGER_OPTIONS 4
PGM_P const state_flanger_entries[] PROGMEM = {
    str_state,
    str_delay,
    str_depth,
    str_rate
};

static void state_flanger_print_option()
{
    uint8_t offset = 2 * (option_pos == option_selected);
    strcpy_P(buffer, (PGM_P)pgm_read_word(&(state_flanger_entries[option_pos])));
    offset += strlen(buffer);

    LCD_printAt(LCD_INSTR_nextLine, LCD_EMPTY_LINE);
    LCD_printAt(LCD_INSTR_nextLine, "> ");
    LCD_printAt(LCD_INSTR_nextLine + 2 * (option_pos == option_selected), buffer);

    if (option_pos == 0) {
        if (active_effects & _BV(EFFECT_FLANGER)) {
            strcpy_P(buffer, str_on);
            LCD_printAt(LCD_INSTR_nextLine + offset, buffer);
        }
        else {
            strcpy_P(buffer, str_off);
            LCD_printAt(LCD_INSTR_nextLine + offset, buffer);
        }
    } else if (option_pos == 1) {
        sprintf(buffer, "%d%%", (uint16_t)(es_flanger_delay * 100));
        LCD_printAt(LCD_INSTR_nextLine + offset, buffer);        
    } else if (option_pos == 2) {
        sprintf(buffer, "%d%%", (uint16_t)(es_flanger_depth * 100));
        LCD_printAt(LCD_INSTR_nextLine + offset, buffer);
    } else if (option_pos == 3) {
        sprintf(buffer, "%dHz", es_flanger_rate_Hz);
        LCD_printAt(LCD_INSTR_nextLine + offset, buffer);
    }
}

static void state_flanger_start()
{
    LCD_writeInstr(LCD_INSTR_clearDisplay);
    LCD_printAt(0, "=====Flanger====");
    g_state = STATE_FLANGER;
    option_pos = 0;
    option_selected = -1;
    state_flanger_print_option();
}

static void state_flanger_hnd(enum MENU_INPUT input)
{
    switch (input) {
        case MENU_INPUT_OK:
            if (option_selected != -1)
                option_selected = -1;
            else
                option_selected = option_pos;
            state_flanger_print_option();
            break;
        case MENU_INPUT_BACK:
            if (option_selected != -1) {
                option_selected = -1;
                state_flanger_print_option();
            } else
                state_all_entries_start(0);
            break;
        case MENU_INPUT_UP:
            if (option_selected == -1) {
                option_pos--;
                if (option_pos == -1)
                    option_pos++;
            } else {
                if (option_selected == 0) {
                    active_effects ^= _BV(EFFECT_FLANGER);
                } else if (option_selected == 1) {
                    increment_float(&es_flanger_delay, 
                        -FLOAT_INCREMENT, 0.0f, 1.0f);                    
                } else if (option_selected == 2)
                    increment_float(&es_flanger_depth, 
                        -FLOAT_INCREMENT, 0.0f, 1.0f);
                else if (option_selected == 3) {
                    increment_uint(&es_flanger_rate_Hz, -1, 0, 16);
                    timer0_top = TIMER_1HZ / es_flanger_rate_Hz;
                }
            }
            state_flanger_print_option();
            break;
        case MENU_INPUT_DOWN:
            if (option_selected == -1) {
                option_pos++;
                if (option_pos == FLANGER_OPTIONS)
                    option_pos--;
            } else {
                if (option_selected == 0) {
                    active_effects ^= _BV(EFFECT_FLANGER);
                } else if (option_selected == 1) {
                    increment_float(&es_flanger_delay, 
                        FLOAT_INCREMENT, 0.0f, 1.0f);
                } else if (option_selected == 2)
                    increment_float(&es_flanger_depth, 
                        FLOAT_INCREMENT, 0.0f, 1.0f);
                else if (option_selected == 3) {
                    increment_uint(&es_flanger_rate_Hz, 1, 0, 16);
                    timer0_top = TIMER_1HZ / es_flanger_rate_Hz;
                }
            }
            state_flanger_print_option();
            break;
    }
}

#define ECHO_OPTIONS 3
PGM_P const state_echo_entries[] PROGMEM= {
    str_state,
    str_delay,
    str_depth
};

static void state_echo_print_option()
{
    uint8_t offset = 2 * (option_pos == option_selected);
    strcpy_P(buffer, (PGM_P)pgm_read_word(&(state_echo_entries[option_pos])));
    offset += strlen(buffer);

    LCD_printAt(LCD_INSTR_nextLine, LCD_EMPTY_LINE);
    LCD_printAt(LCD_INSTR_nextLine, "> ");
    LCD_printAt(LCD_INSTR_nextLine + 2 * (option_pos == option_selected), buffer);

    if (option_pos == 0) {
        if (active_effects & _BV(EFFECT_ECHO)) {
            strcpy_P(buffer, str_on);
            LCD_printAt(LCD_INSTR_nextLine + offset, buffer);
        }
        else {
            strcpy_P(buffer, str_off);
            LCD_printAt(LCD_INSTR_nextLine + offset, buffer);
        }
    } else if (option_pos == 1) {
        sprintf(buffer, "%dms", es_echo_delay);
        LCD_printAt(LCD_INSTR_nextLine + offset, buffer);        
    } else if (option_pos == 2) {
        sprintf(buffer, "%d%%", (uint16_t)(es_echo_depth * 100));
        LCD_printAt(LCD_INSTR_nextLine + offset, buffer);
    }
}


static void state_echo_start()
{
    LCD_writeInstr(LCD_INSTR_clearDisplay);
    LCD_printAt(0, "======Echo======");
    g_state = STATE_ECHO;
    option_pos = 0;
    option_selected = -1;
    state_echo_print_option();
}

static void state_echo_hnd(enum MENU_INPUT input)
{
    switch (input) {
        case MENU_INPUT_OK:
            if (option_selected != -1)
                option_selected = -1;
            else
                option_selected = option_pos;
            state_echo_print_option();
            break;
        case MENU_INPUT_BACK:
            if (option_selected != -1) {
                option_selected = -1;
                state_echo_print_option();
            } else
                state_all_entries_start(0);
            break;
        case MENU_INPUT_UP:
            if (option_selected == -1) {
                option_pos--;
                if (option_pos == -1)
                    option_pos++;
            } else {
                if (option_selected == 0) {
                    active_effects ^= _BV(EFFECT_ECHO);
                } else if (option_selected == 1) {
                    increment_uint(&es_echo_delay, 
                        -20, 0, 790);
                } else if (option_selected == 2)
                    increment_float(&es_echo_depth, 
                        -FLOAT_INCREMENT, 0.0f, 1.0f);
            }
            state_echo_print_option();
            break;
        case MENU_INPUT_DOWN:
            if (option_selected == -1) {
                option_pos++;
                if (option_pos == ECHO_OPTIONS)
                    option_pos--;
            } else {
                if (option_selected == 0) {
                    active_effects ^= _BV(EFFECT_ECHO);
                } else if (option_selected == 1) {
                    increment_uint(&es_echo_delay, 
                        20, 0, 790);
                } else if (option_selected == 2)
                    increment_float(&es_echo_depth, 
                        FLOAT_INCREMENT, 0.0f, 1.0f);
            }
            state_echo_print_option();
            break;
    }
}


#define COMPRESSION_OPTIONS 5
PGM_P const state_compression_entries[] PROGMEM= {
    str_state,
    str_tresh,
    str_kmin,
    str_attack,
    str_release
};

static void state_compression_print_option()
{
    uint8_t offset = 2 * (option_pos == option_selected);
    strcpy_P(buffer, (PGM_P)pgm_read_word(&(state_compression_entries[option_pos])));
    offset += strlen(buffer);

    LCD_printAt(LCD_INSTR_nextLine, LCD_EMPTY_LINE);
    LCD_printAt(LCD_INSTR_nextLine, "> ");
    LCD_printAt(LCD_INSTR_nextLine + 2 * (option_pos == option_selected), buffer);

    if (option_pos == 0) {
        if (active_effects & _BV(EFFECT_COMPRESSION)) {
            strcpy_P(buffer, str_on);
            LCD_printAt(LCD_INSTR_nextLine + offset, buffer);
        }
        else {
            strcpy_P(buffer, str_off);
            LCD_printAt(LCD_INSTR_nextLine + offset, buffer);
        }
    } else if (option_pos == 1) {
        sprintf(buffer, "%d", (uint16_t)(es_compression_knee_treshold * 100));
        LCD_printAt(LCD_INSTR_nextLine + offset, buffer);        
    } else if (option_pos == 2) {
        sprintf(buffer, "%d", (uint16_t)(es_compression_kmin * 100));
        LCD_printAt(LCD_INSTR_nextLine + offset, buffer);
    } else if (option_pos == 3) {
        sprintf(buffer, "%d%%", (uint16_t)(es_compression_alpha_attack * 100));
        LCD_printAt(LCD_INSTR_nextLine + offset, buffer);
    } else if (option_pos == 4) {
        sprintf(buffer, "%d%%", (uint16_t)(es_compression_alpha_release * 100));
        LCD_printAt(LCD_INSTR_nextLine + offset, buffer);
    }
}


static void state_compression_start()
{
    LCD_writeInstr(LCD_INSTR_clearDisplay);
    LCD_printAt(0, "===Compression==");
    g_state = STATE_COMPRESSION;
    option_pos = 0;
    option_selected = -1;
    state_compression_print_option();
}

static void state_compression_hnd(enum MENU_INPUT input)
{
    switch (input) {
        case MENU_INPUT_OK:
            if (option_selected != -1)
                option_selected = -1;
            else
                option_selected = option_pos;
            state_compression_print_option();
            break;
        case MENU_INPUT_BACK:
            if (option_selected != -1) {
                option_selected = -1;
                state_compression_print_option();
            } else
                state_all_entries_start(0);
            break;
        case MENU_INPUT_UP:
            if (option_selected == -1) {
                option_pos--;
                if (option_pos == -1)
                    option_pos++;
            } else {
                if (option_selected == 0) {
                    active_effects ^= _BV(EFFECT_COMPRESSION);
                } else if (option_selected == 1)
                    increment_float(&es_compression_knee_treshold, 
                        -FLOAT_INCREMENT, 0.0f, 1.0f);
                else if (option_selected == 2)
                    increment_float(&es_compression_kmin, 
                        -FLOAT_INCREMENT, 0.0f, 1.0f);
                else if (option_selected == 3)
                    increment_float(&es_compression_alpha_attack, 
                        -FLOAT_INCREMENT, 0.0f, 1.0f);
                else if (option_selected == 4)
                    increment_float(&es_compression_alpha_release, 
                        -FLOAT_INCREMENT, 0.0f, 1.0f);
            }
            state_compression_print_option();
            break;
        case MENU_INPUT_DOWN:
            if (option_selected == -1) {
                option_pos++;
                if (option_pos == COMPRESSION_OPTIONS)
                    option_pos--;
            } else {
                if (option_selected == 0) {
                    active_effects ^= _BV(EFFECT_COMPRESSION);
                } else if (option_selected == 1)
                    increment_float(&es_compression_knee_treshold, 
                        FLOAT_INCREMENT, 0.0f, 1.0f);
                else if (option_selected == 2)
                    increment_float(&es_compression_kmin, 
                        FLOAT_INCREMENT, 0.0f, 1.0f);
                else if (option_selected == 3)
                    increment_float(&es_compression_alpha_attack, 
                        FLOAT_INCREMENT, 0.0f, 1.0f);
                else if (option_selected == 4)
                    increment_float(&es_compression_alpha_release, 
                        FLOAT_INCREMENT, 0.0f, 1.0f);
            }
            state_compression_print_option();
            break;
    }
}
