#include "twi.h"


void TWI_init(void)
{
    // SCL to 400kHz
    TWSR = 0x00;
    TWBR = 12;

    // enable TWI
    TWCR = _BV(TWEN);
}

void TWI_start(void)
{
    TWCR = _BV(TWINT) | _BV(TWSTA) | _BV(TWEN);
    while (!(TWCR & _BV(TWINT)));
}

void TWI_stop(void)
{
    TWCR = _BV(TWINT) | _BV(TWSTO) | _BV(TWEN);
}

void TWI_write(uint8_t data)
{
    TWDR = data;
    TWCR = _BV(TWINT) | _BV(TWEN);
    while (!(TWCR & _BV(TWINT)));
}

uint8_t TWI_read_ack(void)
{
    TWCR = _BV(TWINT) | _BV(TWEN) | _BV(TWEA);
    while ((TWCR & (1<<TWINT)) == 0);
    return TWDR;
}

#define TWI_status() (uint8_t)(TWSR & 0xF8)

// fast mode I2C write
uint8_t DAC_output(uint16_t data)
{
    TWI_start();
    if (TWI_status() != 0x08)
        return 1;
    
    //send first byte
    TWI_write(TWI_1st_BYTE);
    if (TWI_status() != 0x18)
        return 1;

    TWI_write((uint8_t)((data >> 8) & 0x0F));
    if (TWI_status() != 0x28)
        return 1;
    TWI_write((uint8_t)data);
    if(TWI_status() != 0x28)
        return 1;
    
    TWI_stop();
    return 0;
}